/* Copyright (c) 2004 Alessandro Rubini -- GNU GPL 2 or later version */
#include <linux/module.h>
#include <linux/moduleparam.h>
#include <linux/init.h>
#include <linux/delay.h>
#include <linux/gfp.h>
#include <linux/fs.h>
#include <linux/miscdevice.h>
#include <linux/mm.h>
#include <linux/interrupt.h>
#include <asm/timex.h>

int nsec = 0;
module_param(nsec, int, S_IRUGO);
int docli = 1;
module_param(docli, int, S_IRUGO);

unsigned long tc_page;

struct page * tc_nopage(struct vm_area_struct *vma,
			unsigned long addr, int *type)
{
	struct page *page = virt_to_page(tc_page); /* sloppy */
	get_page(page);
	return page;
}

struct vm_operations_struct tc_vmops = {
	.nopage = tc_nopage,
};

int tc_mmap(struct file *file, struct vm_area_struct *vma)
{
	vma->vm_ops = &tc_vmops;
	return 0; /* sloppy */
}

struct file_operations tc_fops = {
	.mmap = tc_mmap,
};

struct miscdevice tc_misc = {
	.minor = MISC_DYNAMIC_MINOR,
	.name = "timecheck",
	.fops = &tc_fops,
};

int __init tc_init(void)
{
	int ret, i;
	u32 *array;

	tc_page = __get_free_page(GFP_KERNEL);
	if (!tc_page) return -ENOMEM;
	array = (u32 *)tc_page;

	ret = misc_register(&tc_misc);
	if (ret) {
		free_page(tc_page);
		return ret;
	}

	/* now fill the array */
	for (i = 0; i < PAGE_SIZE / sizeof(u32); i++) {
		u32 t0, t1;

		if (docli) local_irq_disable();
		t0 = (u32) get_cycles();
		if (nsec) ndelay(i);
		else udelay(i);
		t1 = (u32) get_cycles();
		local_irq_enable();

		array[i] = (t1 - t0)/((cpu_khz+500)/1000);
		if (nsec) array[i] = t1 - t0;
		schedule();
	}
	return 0;
}

void __exit tc_exit(void)
{
	free_page(tc_page);
	misc_deregister(&tc_misc);
}

module_init(tc_init);
module_exit(tc_exit);













